@@ -116,7 +116,7 @@ def evaluate(adata: ad.AnnData,
         ebm = k_bet = None
 
     # plot UMAP embeddings
-    if draw:
+    if draw and (return_fig or plot_dir is not None):
         if color_by is None:
             color_by = [batch_col, cell_type_col] if need_batch else [cell_type_col]
         if cluster_key is not None:
@@ -227,6 +227,7 @@ def calculate_kbet(
     _logger.info('Calculating kbet...')
     assert batch_col in adata.obs
     if adata.obs[batch_col].dtype.name != "category":
+        _logger.warning(f'Making the column {batch_col} of adata.obs categorical.')
         adata.obs[batch_col] = adata.obs[batch_col].astype('category')
 
     ideal_dist = (
